/* jinx.c v.1.0b Altair BDSC         */
/* Written by Wayne Parham           */ 
/* 07/14/2021  v.1.0a                */
/* 08/28/2022  v.1.0b                */


#include <stdio.h>
#include "jinx.h"


int main() {
   initialize();
   clearScreen();
   printf( s, introText );
   look();

   while( running() ) {
      prompt();
      playerAction();
      checkFate();
   }

   return( NORMAL );
}


void initialize() {

   setmem( buffer, SCREENWIDTH, 0 );
   setmem( visited, sizeof(visited[0])*(LastLocation+1), 0 );
   setmem( Inventory, sizeof(Inventory[0])*(LastItem+1), 0 );
#ifdef ENBLSAVE
   setmem( &saveState, sizeof(saveState), 0 );
#endif
   initFormats();
   initResponses();
   initItmDesc();
   initItems();
   initLocs();
   initLocDets();
   initLocations();
   initDirections();
   initMessages();
   initMap1();
   initMap2();

   WorldState = 1;
   turnsPlayed = 0;
   hoursPassed = 0;
   minutesPassed = 0;
   turnCandle = 0;
   turnBell = 0;
   turnBook = 0;
   wraithEncounters = 0;
   wraithStrength = INITWRSTRENGTH;
   wraithAgitation = INITWRAGITATION;
   turnsToAgitate = DEFAULTAGITATEFREQUENCY;

   Move[30][North] = 0;                                              /* 3017 Front Door */
   Move[37][North]  = 0;                                             /* 3017 Shelter    */
   Move[38][South] = 0;
   Move[78][Down] = 0;                                               /* 4735 Garage     */
   Move[140][Up] = 0;
   Move[121][Down] = 0;                                              /* Forest Grate    */
   Move[142][Up] = 0;

}


int strncasecmp(s1, s2, length)                                      /* String comparison */
    char* s1;
    char* s2;
    int length;
{
    char* c1;
    char* c2;
    int   l;
    int   r;

    c1 = s1;
    c2 = s2;
    l  = 0;
    r  = 0;

    while( (r == 0) && (l < length) && (*c2 != '\0') ) {
       r = *c1 - *c2;
       c1++;
       c2++;
       l++;
    }

    return( r );
}


void clearScreen() {                                                 /* Clear screen */
   int i;

   for( i = 0; i < SCREENHEIGHT; i++ )
      printf( Cr );
}


void timeElapsed( minutes ) {                                        /* Record passage of time */
   if( minutes > 0 ) {
      minutesPassed += minutes;
      if( minutesPassed > 60 ) {
         hoursPassed++;
         minutesPassed -= 60;
      }
   } else {
      minutesPassed -= minutes;
      if( (minutesPassed < 0) && (hoursPassed > 0) ) {
         hoursPassed--;
      }
   }
}


void bufferToLower() {                                               /* Convert input buffer to lowercase */
   int i;

   for( i=0; buffer[i]; i++ )
      buffer[i] = tolower( buffer[i] );
}


void setState( state ) int state; {                                  /* Sets WorldState bit */
   WorldState = WorldState | (1 << state);
} 


void clearState( state ) int state; {                                /* Clears WorldState bit */
   WorldState = WorldState & ~(1 << state);
}


int getState( state ) int state; {                                   /* Get WorldState bit */
   if( ( (WorldState) >> (state) ) & 1 )
      return( TRUE );
   else
      return( FALSE );
}


int running() {                                                      /* Check to see if game over */
   return( getState(0) );
}


#ifdef ENBLSAVE
void loadGameState() {                                               /* Load Game State */
   int i;
   FILE *loadFile;

   loadFile = fopen( "jinx.sav", "rb" );

   if( loadFile == NULL ) {
      printf( "\nError opening jinx.sav file.  Perhaps one was never created.\n\n" );   
      return;             
   } else {
      fread( &saveState, sizeof(saveState), 1, loadFile ); 
      if( ferror ( loadFile ) ) {
         printf( "\nError reading jinx.sav file.\n\n" );
      } else {

         currentLocation  = saveState.curLoc;
         turnsPlayed      = saveState.turnsP;
         turnFlood        = saveState.turnF;
         turnCandle       = saveState.turnC;
         turnBell         = saveState.turnR;
         turnBook         = saveState.turnB;
         minutesPassed    = saveState.minsPassed;
         hoursPassed      = saveState.hrsPassed;
         wraithEncounters = saveState.wraithE;
         wraithStrength   = saveState.wraithS;
         wraithAgitation  = saveState.wraithA;
         turnsToAgitate   = saveState.turnsTA;
         Move[30][North]  = saveState.Move30N;
         Move[37][North]  = saveState.Move37N;
         Move[38][South]  = saveState.Move38S;
         Move[78][Down]   = saveState.Move78D;
         Move[140][Up]    = saveState.Move140U;
         Move[121][Down]  = saveState.Move121D;
         Move[142][Up]    = saveState.Move142U;

         for( i=1; i<=LastLocation; i++ ) { 
            visited[i]    = saveState.vis[i];
         }

         for( i=1; i<=LastItem; i++ ) {
            Inventory[i]  = saveState.inv[i];
         }

         for( i=1; i<=LastItem; i++ ) {
            locationOfItem[i] = saveState.loc[i];
         }

         if( flooded() ) {                                           /* If saved game had flooded */
            for( i = 29; i < 38; i++ ) {                             /* then remove house 3017    */
               Location[i] = muddyarea;
               Descript[i] = NULL;
            }
            Descript[31] = scattereddebris;                          /* and litter with debris    */
         }

         printf( "\nPrevious game restored.\n" );
      }

      fclose( loadFile );
   }

   return;
}


void saveGameState() {                                               /* Save Game State */
   int i;
   FILE *saveFile;

   saveFile = fopen( "jinx.sav", "wb+" );

   if( saveFile == NULL ) {
      printf( "\nError creating jinx.sav file.\n\n" );   
      return;             
   } else {

      saveState.curLoc       = currentLocation;
      saveState.turnsP       = turnsPlayed;
      saveState.turnF        = turnFlood;
      saveState.turnC        = turnCandle;
      saveState.turnR        = turnBell;
      saveState.turnB        = turnBook;
      saveState.minsPassed   = minutesPassed;
      saveState.hrsPassed    = hoursPassed;
      saveState.wraithE      = wraithEncounters;
      saveState.wraithS      = wraithStrength;
      saveState.wraithA      = wraithAgitation;
      saveState.turnsTA      = turnsToAgitate;
      saveState.Move30N      = Move[30][North];
      saveState.Move37N      = Move[37][North];
      saveState.Move38S      = Move[38][South];
      saveState.Move78D      = Move[78][Down];
      saveState.Move140U     = Move[140][Up];
      saveState.Move121D     = Move[121][Down];
      saveState.Move142U     = Move[142][Up];

      for( i=1; i<=LastLocation; i++ ) { 
         saveState.vis[i]    = visited[i];
      }

      for( i=1; i<=LastItem; i++ ) {
         saveState.inv[i]    = Inventory[i];
      }

      for( i=1; i<=LastItem; i++ ) {
         saveState.loc[i]    = locationOfItem[i];
      }

      fwrite( &saveState, sizeof(saveState), 1, saveFile );
      if( ferror ( saveFile ) ) {
         printf( "\nError writing jinx.sav file.\n\n" );
      } else {
         printf( "\nSaved.\n" );
      }

      fclose( saveFile );
   }

   return;
}
#endif