/* Mand.c                                        */
/*                                               */
/* Generates ASCII output of the Mandelbrot set  */
/* using (long) 32-bit integer math.             */
/*                                               */
/* Written by Wayne Parham                       */


#include "stdio.h"


#define INIT_LEFT     -560  /* (x size 800, 80 steps) */
#define INIT_RIGHT     240
#define INIT_TOP       220  /* (y size 460, 23 steps) */
#define INIT_BOTTOM   -240
#define INIT_X_INCR     10  /* (step ratio 2:1, y:x ) */
#define INIT_Y_INCR     20
#define INIT_MAX_ITER   20


void intro() {
   puts( "\n" );
   puts( "The Mandelbrot set includes all two-dimensional c values for which the function" );
   puts( "z = z^2 + c does not diverge when iterated from z = 0.  It was named in tribute" );
   puts( "to the mathematician Benoit Mandelbrot, a pioneer of fractal geometry.   He was" );
   puts( "first  to see a visualization of the set on March 1, 1980, at IBM's  Thomas  J." );
   puts( "Watson Research Center in Yorktown Heights, New York.\n" );
   puts( "The  Mandelbrot  set  has  its  origin  in  complex  dynamics,  a  field  first" );
   puts( "investigated by the French mathematicians Pierre Fatou and Gaston Julia at  the" );
   puts( "beginning of the 20th century. This fractal was first defined and drawn in 1978" );
   puts( "by Robert W. Brooks and Peter Matelski as part of a study of Kleinian groups.\n" );
   puts( "Mandelbrot  studied the parameter space of quadratic polynomials in an  article" );
   puts( "that  appeared  in 1980.  The mathematical study of the Mandelbrot  set  really" );
   puts( "began with work by the mathematicians Adrien Douady and John H. Hubbard (1985)," );
   puts( "who  established many of its fundamental properties and named the set in  honor" );
   puts( "of Mandelbrot for his influential work in fractal geometry.\n" );
   puts( "This  program  displays the set on an ASCII terminal, much  like  the  earliest" );
   puts( "computers did.  It is a throwback to the computing technologies of  yesteryear.\n" );
}


void Mandelbrot() {
   long boundL      =  INIT_LEFT;      /* -560  (x size 800, 80 steps) */
   long boundR      =  INIT_RIGHT;     /*  240                         */
   long boundT      =  INIT_TOP;       /*  220                         */
   long boundB      =  INIT_BOTTOM;    /* -240  (y size 460, 23 steps) */
   long x_iter      =  INIT_X_INCR;    /*  10                          */
   long y_iter      =  INIT_Y_INCR;    /*  20   (step ratio 2:1, y:x)  */
   long max         =  INIT_MAX_ITER;  /*  20                          */
   long n,zxx,zyy,zx,zy,cx,cy;
   char p;

   for( cy=boundT; cy > boundB; cy -= y_iter ) {
      putchar( '\n' );
      for( cx=boundL; cx < boundR; cx += x_iter ) {
         n  = 0;
         zy = 0;
         zx = 0;
         p  = ' ';
         while( n < max ) {
            zxx = zx * zx / 200;
            zyy = zy * zy / 200;
            if( (zxx + zyy) > 800 ) {
               if( n > 9 ) 
                  p = '0';
               else
                  p = n + 48;
               n = max;
            }
            zy = zx * zy / 100 + cy;
            zx = zxx - zyy + cx;
            n++;
         }
         putchar( p );
      }
   }
   return;
}


void pressKey() {
   puts( "Press any key to continue.\n" );
   getchar();
}


void main() {
   intro();
   pressKey();
   Mandelbrot();
}
